import numpy
import constant


def advection(plon, plat, lon, lat, u, v, tadv, dt):
    # Initialize trajectory lists
    traj_time = [0]
    traj_lon = [plon]
    traj_lat = [plat]
    time = 0

    # Convert dt in seconds
    dt_s = dt * constant.daytosec
    
    # Velocity not defined is 0
    u[~numpy.isfinite(u)] = 0
    v[~numpy.isfinite(v)] = 0
    dlon = abs(lon[0, 1] - lon[0, 0])
    dlat = abs(lat[1, 0] - lat[0, 0])
    #Loop on Advection time
    while time < tadv:
        time += dt
        dist = (((lon - plon)*numpy.cos(numpy.deg2rad(lat)))**2
                + (lat - plat)**2)
        dist_min = numpy.min(dist)
        if len(u.shape) > 2:
            u_time0 = u[numpy.max(round(time)), :, :] #u.shape-1]
            v_time0 = v[numpy.max(round(time)), :, :] # v.shape-1]
        else:
            u_time0 = u
            v_time0 = v
        # Convert velocity in deg/s:
        u_time, v_time = converttoangular(lon, lat, numpy.array(u_time0), numpy.array(v_time0))
        u_time[:, 0] = 0
        u_time[:, -1] = 0
        v_time[0, :] = 0
        v_time[-1, :] = 0
        #u_time = u_time / constant.degtom #(constant.degtom * numpy.cos(numpy.deg2rad(lat)))
        #v_time = v_time / constant.degtom
        
        i, j = numpy.where(dist == dist_min)
        if isinstance(i, (list, tuple, numpy.ndarray)): 
            i = i[0]
            j = j[0]
        lon_dist0 = (lon[i, j] - plon) / dlon
        if lon_dist0 > 0:
            lon_dist1 = 1 - lon_dist0
            j1 = j + 1
        else:
            lon_dist1 = abs(lon_dist0)
            lon_dist0 = 1 - abs(lon_dist0)
            j1 = j
            j = j - 1
        lat_dist0 = (lat[i, j] - plat) / dlat
        if lat_dist0 > 0:
            lat_dist1 = 1 - lat_dist0
            i1 = i + 1
        else:
            lat_dist1 = abs(lat_dist0)
            lat_dist0 = 1 - abs(lat_dist0)
            i1 = i
            i = i - 1
        u_mean = u_time[i:(i1 + 1), j] * lon_dist0 + u_time[i:(i1 + 1), j1] * lon_dist1
        u_mean2 = u_mean[0] * lat_dist0 + u_mean[1] * lat_dist1

        v_mean = v_time[i:(i1 + 1), j] * lon_dist0 + v_time[i:(i1 + 1), j1] * lon_dist1
        v_mean2 = v_mean[0] * lat_dist0 + v_mean[1] * lat_dist1

        plon = plon + u_mean2 * dt_s
        plat = plat + v_mean2 * dt_s
        if (plon == traj_lon[-1]) or (plat == traj_lat[-1]):
            print('coast reached')
            break
        #except: print(plon, traj_lon[-1], plat, traj_lat[-1])
        traj_lon.append(plon)
        traj_lat.append(plat)
        traj_time.append(time)
        perc = (time / tadv * 100)
        if perc%10 == 0:
            print(perc, '% done')
    return traj_lon, traj_lat, traj_time


def tconcatenate(listu, listv, time):
    ut = []
    vt = []
    for t in range(time):
        ut.append(u)
        vt.append(v)
    return ut, vt


def advect_ensemble_point(grid_x, lon, lat, u, v):
    xt=[]
    yt=[]
    for x, y in enumerate(grid_x[0], grid_x[1]):
        xtmp, ytmp = advection(x, y, lon, lat, u, v)
        xt.append(x)
        yt.append(y)
    return xt, yt

def converttoangular(x, y, u, v):
    import constant
    """
    Convert U V components from metric to angular system units                  
    """                                                                         
    assert(u.shape == v.shape)                                                  
                                                                                
    nx = len(x)                                                                 
    ny = len(y)                                                                 
                                                                                
    # Keep longitude between -180, 180                                          
    x[x > 180] -= 360                                                           
    x0 = x                                                                      
    y0 = y                                                                      
                                                                          
    # Conversion from spherical to cartesian coordinates and move it            
    # for one second using U and V component                                    
    lon = numpy.radians(x0)                                                     
    lat = numpy.radians(y0)                                                     
    sin_x = numpy.sin(lon)                                                      
    cos_x = numpy.cos(lon)                                                      
    sin_y = numpy.sin(lat)                                                      
    cos_y = numpy.cos(lat)                                                      
    xc = -u * sin_x - v * cos_x * sin_y                                         
    yc = u * cos_x - v * sin_y * sin_x                                          
    zc = v * cos_y                                                              
    xc += constant.Rearth * cos_y * cos_x                                          
    yc += constant.Rearth * cos_y * sin_x                                          
    zc += constant.Rearth * sin_y                                                  
                                                                                
    # Conversion from cartesian to spherical coordinates                        
    x1 = numpy.degrees(numpy.arctan2(yc, xc))                                   
    y1 = numpy.degrees(numpy.arcsin(                                            
                       zc / numpy.sqrt(xc * xc + yc * yc + zc * zc)))           
    dx = x1 - x0                                                                
    dy = y1 - y0                                                                
                                                                                
    # Return the velocity in degrees/s                                          
    return (dx + 180) % 360 - 180, dy                                           
                                       